==> nhttpd-0.0.3.tar.gz: nhttpd-0.0.3/src/config.c <==
#include <header.h>

void config_cleanup( struct config_t *config )
{
	if ( config->port != NULL )
		free( config->port );
	if ( config->file_config != NULL )
		free( config->file_config );
	if ( config->directory_index_file != NULL )
		free( config->directory_index_file );
	if ( config->vhost != NULL )
		free( config->vhost );
	xmlCleanupParser();
}

int parse_config( const int argc, char * const argv[], struct config_t *config )
{
	int				c;		//getopt
	extern char			*optarg;	//getopt
	extern int			optind;		//getopt

	xmlDocPtr			doc;
	xmlNode				*node_server;
	xmlNode				*subnode;
	xmlNode				*subsubnode;
	char				*value;
	int				value_len;

	extern int			terminate;	//declare in main.c
	extern struct log_t		tlog;

	optind = 1;

	memset( config, 0, sizeof(struct config_t) );

	config->file_config = malloc( sizeof(char) * 1024 );
	strncpy( config->file_config, DEFAULT_CONFIG_FILE_CONFIG, 1024 );

	while ( (c = getopt(argc, argv, "c:h")) > 0 )
	{
		switch ( c )
		{
			case 'c':
				config->file_config = strncpy( config->file_config, optarg, 1024 );
			break;
			case 'h':
				printf("Usage: midware1 [-c configfile]\n");
				terminate = TRUE;
				free( config->file_config );
				return 0;
			break;
		}
	}

	doc = xmlParseFile(config->file_config);

	if ( doc == NULL )
	{
		free( config->file_config );
		return -1;
	}

	/* server configuration begin */
	node_server = xml_find_doc_child_by_tag( doc, "Server" );

	if ( node_server == NULL )
	{
		config_cleanup( config );
		return -1;
	}

	config->listen_backlog =	xml_get_config_int_value( node_server, "ListenBacklog", DEFAULT_CONFIG_LISTEN_BACKLOG );
	config->timeout =		xml_get_config_int_value( node_server, "TimeOut", DEFAULT_CONFIG_TIMEOUT );
	config->thread_pool_size =	xml_get_config_int_value( node_server, "ThreadPoolSize", DEFAULT_CONFIG_THREAD_POOL_SIZE );
	if ( config->thread_pool_size < 4 )
		config->thread_pool_size = 4;

	value =				xml_get_config_char_value( node_server, "BufferSize", DEFAULT_CONFIG_DEFAULT_BUF_SIZE );
	value_len = strlen( value );
	switch ( value[value_len-1] )
	{
		case 'k':
		case 'K':
			config->default_buffer_size = atoi( value ) * 1024;
		break;
		case 'm':
		case 'M':
			config->default_buffer_size = atoi( value ) * 1024 * 1024;
		break;
		case 'g':
		case 'G':
			config->default_buffer_size = atoi( value ) * 1024 * 1024 * 1024;
		break;
		default:
			config->default_buffer_size = atoi( value );
		break;
	}

	if ( config->default_buffer_size <= 0 )
	{
		config_cleanup( config );
		return -1;
	}

	config->file_mime =		xml_get_config_char_value( node_server, "MimeFile", DEFAULT_CONFIG_FILE_MIME );

	config->server_user =		xml_get_config_char_value( node_server, "ServerUser", DEFAULT_CONFIG_SERVER_USER );
	config->document_root =		xml_get_config_char_value( node_server, "DocumentRoot", DEFAULT_CONFIG_DOCUMENT_ROOT );
	config->default_charset =	xml_get_config_char_value( node_server, "DefaultCharSet", DEFAULT_CONFIG_DEFAULT_CHARSET );

	/* port setting start */
	config->port_len = 0;
	subnode = xml_find_child_by_tag( node_server, "Listen" );
	if ( subnode == NULL )
	{
		config->port_len = 1;
		config->port = malloc( sizeof( int ) );
		config->port[0] = DEFAULT_CONFIG_PORT;
	}
	else
	{
		subsubnode = xml_find_child_by_tag( subnode, "Port" );
		config->port_len = 1;
		config->port = malloc( sizeof( int ) );
		if ( subsubnode == NULL )
			config->port[0] = DEFAULT_CONFIG_PORT;
		else
		{
			config->port[0] = atoi( xml_get_config_char_value( subsubnode, NULL, NULL ) );
			for ( subsubnode = xml_find_next_by_tag( subsubnode, "Port" ); subsubnode; subsubnode = xml_find_next_by_tag( subsubnode, "Port" ) )
			{
				config->port = realloc( config->port, sizeof(int) * (config->port_len + 1) );
				config->port[config->port_len] = atoi(xml_get_config_char_value( subsubnode, NULL, NULL ));
				config->port_len++;
			}
		}
	}
	/* port setting end */

	/* document_root & directory index setting start */
	config->directory_index_len = 0;
	subnode = xml_find_child_by_tag( node_server, "DirectoryIndex" );
	if ( subnode == NULL )
	{
		config->directory_index_len = 1;
		config->directory_index_file = malloc( sizeof(char *) * 1 );
		config->directory_index_file[0] = DEFAULT_CONFIG_DIRECTORY_INDEX;
	}
	else
	{
		subsubnode = xml_find_child_by_tag( subnode, "Filename" );
		config->directory_index_len = 1;
		config->directory_index_file = malloc( sizeof(char *) * 1 );
		if ( subsubnode == NULL )
			config->directory_index_file[0] = DEFAULT_CONFIG_DIRECTORY_INDEX;
		else
		{
			config->directory_index_file[0] = xml_get_config_char_value( subsubnode, NULL, NULL );
			for ( subsubnode = xml_find_next_by_tag( subsubnode, "Filename" ); subsubnode; subsubnode = xml_find_next_by_tag( subsubnode, "Filename" ) )
			{
				config->directory_index_file = realloc( config->directory_index_file, sizeof(char *) * (config->directory_index_len + 1) );
				config->directory_index_file[config->directory_index_len] = xml_get_config_char_value( subsubnode, NULL, NULL );
				config->directory_index_len++;
			}
		}
	}
	/* document_root & directory index setting end */

	/* log setting start */
	memset( &tlog, 0, sizeof(struct log_t) );
	pthread_mutex_init( &tlog.mutex, NULL );
	subnode = xml_find_child_by_tag( node_server, "Log" );
	if ( subnode == NULL )
	{
		tlog.path = DEFAULT_CONFIG_LOG_PATH;
	}
	else
	{
		tlog.path =		xml_get_config_char_value( subnode, "Path", DEFAULT_CONFIG_LOG_PATH );
		tlog.rotate =		xml_get_config_boolean_value( subnode, "Rotate", FALSE );
		tlog.vhost_log_split =	xml_get_config_boolean_value( subnode, "VirtualHostLogSplit", FALSE );
		if ( tlog.rotate == TRUE )
		{
			value =		xml_get_config_char_value( subnode, "RotateCycle", "0" );
			tlog.rotate_cycle = atoi( value );
			switch( value[strlen(value) - 1] )
			{
				case 'H':
					tlog.rotate_cycle = tlog.rotate_cycle * 60 * 60;
				break;
				case 'D':
					tlog.rotate_cycle = tlog.rotate_cycle * 60 * 60 * 24;
				break;
				case 'M':
					tlog.rotate_cycle = tlog.rotate_cycle * 60 * 60 * 24 * 30;
				break;
				default:
					tlog.rotate = FALSE;
					tlog.rotate_cycle = 0;
				break;
			}
		}
	}

	tlog.logfile = malloc( sizeof(struct logfile_t) );
	memset( &(tlog.logfile[0]), 0, sizeof(struct logfile_t) );

	pthread_mutex_init( &(tlog.logfile[0].mutex), NULL );
	tlog.logfile[0].is_default = TRUE;
	snprintf( tlog.logfile[0].log_filename_access, LOG_FILENAME_LEN, "%s/access_%%d%%d%%d_%%d.log", tlog.path );
	snprintf( tlog.logfile[0].log_filename_error, LOG_FILENAME_LEN, "%s/error_%%d%%d%%d_%%d.log", tlog.path );

	tlog.logfile_len++;
	/* log setting end */

	/* virtual host setting start */
	config->vhost_len = 0;
	config->vhost = malloc( sizeof( struct vhost_t ) );
	for ( subnode = xml_find_child_by_tag( node_server, "VirtualHost" ); subnode; subnode = xml_find_next_by_tag( subnode, "VirtualHost" ) )
	{
		config->vhost = realloc( config->vhost, sizeof( struct vhost_t ) * (config->vhost_len + 1) );
		config->vhost[config->vhost_len].host =			xml_get_config_char_value( subnode, "Host", NULL );
		config->vhost[config->vhost_len].port =			xml_get_config_int_value( subnode, "Port", config->port[0] );
		config->vhost[config->vhost_len].document_root =	xml_get_config_char_value( subnode, "DocumentRoot", NULL );
		if ( config->vhost[config->vhost_len].host == NULL )
			continue;
		else
			config->vhost[config->vhost_len].host_len = strlen( config->vhost[config->vhost_len].host );

		if ( tlog.vhost_log_split == TRUE )
		{
			tlog.logfile = realloc( tlog.logfile, sizeof(struct logfile_t) * (tlog.logfile_len + 1) );
			memset( &(tlog.logfile[(tlog.logfile_len + 1)]), 0, sizeof(struct logfile_t) );

			pthread_mutex_init( &(tlog.logfile[tlog.logfile_len].mutex), NULL );
			pthread_mutex_init( &(tlog.logfile[tlog.logfile_len].file_mutex), NULL );
			tlog.logfile[tlog.logfile_len].vhost = &(config->vhost[config->vhost_len]);
			snprintf( tlog.logfile[tlog.logfile_len].log_filename_access, LOG_FILENAME_LEN, "%s/access_%s_%%d%%d%%d_%%d.log", tlog.path, config->vhost[config->vhost_len].host );
			snprintf( tlog.logfile[tlog.logfile_len].log_filename_error, LOG_FILENAME_LEN, "%s/error_%s_%%d%%d%%d_%%d.log", tlog.path, config->vhost[config->vhost_len].host );

			tlog.logfile_len++;
		}

		config->vhost_len++;
	}
	/* virtual host setting end */

	return 0;
}

==> nhttpd-0.1.1.tar.gz: nhttpd-0.1.1/src/config.c <==
#include <header.h>

void config_cleanup( struct config_t *config )
{
	if ( config->port != NULL )
		free( config->port );
	if ( config->file_config != NULL )
		free( config->file_config );
	if ( config->directory_index_file != NULL )
		free( config->directory_index_file );
	if ( config->vhost != NULL )
		free( config->vhost );
	xmlCleanupParser();
}

int parse_config( const int argc, char * const argv[], struct config_t *config )
{
	int				c;		//getopt
	extern char			*optarg;	//getopt
	extern int			optind;		//getopt

	xmlDocPtr			doc;
	xmlNode				*node_server;
	xmlNode				*subnode;
	xmlNode				*subsubnode;
	char				*value;
	int				value_len;

	int				i;
	struct vhost_t			*tmp_vhost;
	struct logfile_t		*tmp_logfile;

	extern int			terminate;	//declare in main.c
	extern struct log_t		tlog;


	optind = 1;

	memset( config, 0, sizeof(struct config_t) );

	config->file_config = malloc( sizeof(char) * 1024 );
	strncpy( config->file_config, DEFAULT_CONFIG_FILE_CONFIG, 1024 );

	while ( (c = getopt(argc, argv, "c:hv")) > 0 )
	{
		switch ( c )
		{
			case 'c':
				config->file_config = strncpy( config->file_config, optarg, 1024 );
			break;
			case 'h':
				printf("Usage: midware1 [-c configfile]\n");
				terminate = TRUE;
				free( config->file_config );
				return 0;
			break;
			case 'v':
				printf("Version: %s %s\n", SERVER_NAME, SERVER_VERSION);
				terminate = TRUE;
				free( config->file_config );
				return 0;
			break;
		}
	}

	doc = xmlParseFile(config->file_config);

	if ( doc == NULL )
	{
		free( config->file_config );
		return -1;
	}

	/* server configuration begin */
	node_server = xml_find_doc_child_by_tag( doc, "Server" );

	if ( node_server == NULL )
	{
		config_cleanup( config );
		return -1;
	}

	config->listen_backlog =	xml_get_config_int_value( node_server, "ListenBacklog", DEFAULT_CONFIG_LISTEN_BACKLOG );
	config->timeout =		xml_get_config_int_value( node_server, "TimeOut", DEFAULT_CONFIG_TIMEOUT );
	config->thread_pool_size =	xml_get_config_int_value( node_server, "ThreadPoolSize", DEFAULT_CONFIG_THREAD_POOL_SIZE );
	if ( config->thread_pool_size < 4 )
		config->thread_pool_size = 4;

	value =				xml_get_config_char_value( node_server, "BufferSize", DEFAULT_CONFIG_DEFAULT_BUF_SIZE );
	value_len = strlen( value );
	switch ( value[value_len-1] )
	{
		case 'k':
		case 'K':
			config->default_buffer_size = atoi( value ) * 1024;
		break;
		case 'm':
		case 'M':
			config->default_buffer_size = atoi( value ) * 1024 * 1024;
		break;
		case 'g':
		case 'G':
			config->default_buffer_size = atoi( value ) * 1024 * 1024 * 1024;
		break;
		default:
			config->default_buffer_size = atoi( value );
		break;
	}

	if ( config->default_buffer_size <= 0 )
	{
		config_cleanup( config );
		return -1;
	}

	config->file_mime =		xml_get_config_char_value( node_server, "MimeFile", DEFAULT_CONFIG_FILE_MIME );

	config->server_user =		xml_get_config_char_value( node_server, "ServerUser", DEFAULT_CONFIG_SERVER_USER );
	config->document_root =		xml_get_config_char_value( node_server, "DocumentRoot", DEFAULT_CONFIG_DOCUMENT_ROOT );
	config->default_charset =	xml_get_config_char_value( node_server, "DefaultCharSet", DEFAULT_CONFIG_DEFAULT_CHARSET );

	/* port setting start */
	config->port_len = 0;
	subnode = xml_find_child_by_tag( node_server, "Listen" );
	if ( subnode == NULL )
	{
		config->port_len = 1;
		config->port = malloc( sizeof( int ) );
		if ( config->port == NULL )
		{
			config_cleanup( config );
			return -1;
		}
		config->port[0] = DEFAULT_CONFIG_PORT;
	}
	else
	{
		subsubnode = xml_find_child_by_tag( subnode, "Port" );
		config->port_len = 1;
		config->port = malloc( sizeof( int ) );
		if ( config->port == NULL )
		{
			config_cleanup( config );
			return -1;
		}
		if ( subsubnode == NULL )
			config->port[0] = DEFAULT_CONFIG_PORT;
		else
		{
			config->port[0] = atoi( xml_get_config_char_value( subsubnode, NULL, NULL ) );
			for ( subsubnode = xml_find_next_by_tag( subsubnode, "Port" ); subsubnode; subsubnode = xml_find_next_by_tag( subsubnode, "Port" ) )
			{
				config->port = realloc( config->port, sizeof(int) * (config->port_len + 1) );
				config->port[config->port_len] = atoi(xml_get_config_char_value( subsubnode, NULL, NULL ));
				config->port_len++;
			}
		}
	}
	/* port setting end */

	/* document_root & directory index setting start */
	config->directory_index_len = 0;
	subnode = xml_find_child_by_tag( node_server, "DirectoryIndex" );
	if ( subnode == NULL )
	{
		config->directory_index_len = 1;
		config->directory_index_file = malloc( sizeof(char *) * 1 );
		if ( config->directory_index_file == NULL )
		{
			config_cleanup( config );
			return -1;
		}
		config->directory_index_file[0] = DEFAULT_CONFIG_DIRECTORY_INDEX;
	}
	else
	{
		subsubnode = xml_find_child_by_tag( subnode, "Filename" );
		config->directory_index_len = 1;
		config->directory_index_file = malloc( sizeof(char *) * 1 );
		if ( config->directory_index_file == NULL )
		{
			config_cleanup( config );
			return -1;
		}
		if ( subsubnode == NULL )
			config->directory_index_file[0] = DEFAULT_CONFIG_DIRECTORY_INDEX;
		else
		{
			config->directory_index_file[0] = xml_get_config_char_value( subsubnode, NULL, NULL );
			for ( subsubnode = xml_find_next_by_tag( subsubnode, "Filename" ); subsubnode; subsubnode = xml_find_next_by_tag( subsubnode, "Filename" ) )
			{
				config->directory_index_file = realloc( config->directory_index_file, sizeof(char *) * (config->directory_index_len + 1) );
				config->directory_index_file[config->directory_index_len] = xml_get_config_char_value( subsubnode, NULL, NULL );
				config->directory_index_len++;
			}
		}
	}
	/* document_root & directory index setting end */

	/* log setting start */
	memset( &tlog, 0, sizeof(struct log_t) );
	subnode = xml_find_child_by_tag( node_server, "Log" );
	if ( subnode == NULL )
	{
		tlog.path = DEFAULT_CONFIG_LOG_PATH;
	}
	else
	{
		tlog.path =		xml_get_config_char_value( subnode, "Path", DEFAULT_CONFIG_LOG_PATH );
		tlog.rotate =		xml_get_config_boolean_value( subnode, "Rotate", FALSE );
		tlog.vhost_log_split =	xml_get_config_boolean_value( subnode, "VirtualHostLogSplit", FALSE );
		if ( tlog.rotate == TRUE )
		{
			value =		xml_get_config_char_value( subnode, "RotateCycle", "0" );
			tlog.rotate_cycle = atoi( value );
			switch( value[strlen(value) - 1] )
			{
				case 'H':
					tlog.rotate_cycle = tlog.rotate_cycle * 60 * 60;
				break;
				case 'D':
					tlog.rotate_cycle = tlog.rotate_cycle * 60 * 60 * 24;
				break;
				case 'M':
					tlog.rotate_cycle = tlog.rotate_cycle * 60 * 60 * 24 * 30;
				break;
				default:
					tlog.rotate = FALSE;
					tlog.rotate_cycle = 0;
				break;
			}
		}
	}

	tlog.logfile = malloc( sizeof(struct logfile_t) );
	if ( tlog.logfile == NULL )
	{
		config_cleanup( config );
		return -1;
	}
	memset( &(tlog.logfile[0]), 0, sizeof(struct logfile_t) );

	tlog.logfile[0].is_default = TRUE;
	snprintf( tlog.logfile[0].log_filename_access, LOG_FILENAME_LEN, "%s/access_%%d%%d%%d_%%d.log", tlog.path );
	snprintf( tlog.logfile[0].log_filename_error, LOG_FILENAME_LEN, "%s/error_%%d%%d%%d_%%d.log", tlog.path );

	tlog.logfile_len++;
	/* log setting end */

	/* virtual host setting start */
	config->vhost_len = 0;
	config->vhost = NULL;
	for ( subnode = xml_find_child_by_tag( node_server, "VirtualHost" ); subnode; subnode = xml_find_next_by_tag( subnode, "VirtualHost" ) )
	{
		if ( config->vhost == NULL && config->vhost_len == 0 )
		{
			config->vhost = malloc( sizeof(struct vhost_t) );
			if ( config->vhost == NULL )
			{
				config_cleanup( config );
				return -1;
			}
		}
		else
		{
			tmp_vhost = malloc( sizeof(struct vhost_t) * (config->vhost_len + 1) );
			if ( tmp_vhost == NULL )
			{
				config_cleanup( config );
				return -1;
			}
			tmp_vhost = memcpy( tmp_vhost, config->vhost, sizeof(struct vhost_t) * config->vhost_len );
			memset( &(tmp_vhost[config->vhost_len]), 0, sizeof(struct vhost_t) );
			free( config->vhost );
			config->vhost = tmp_vhost;
		}
		config->vhost[config->vhost_len].host =			xml_get_config_char_value( subnode, "Host", NULL );
		config->vhost[config->vhost_len].port =			xml_get_config_int_value( subnode, "Port", config->port[0] );
		config->vhost[config->vhost_len].document_root =	xml_get_config_char_value( subnode, "DocumentRoot", NULL );
		if ( config->vhost[config->vhost_len].host == NULL )
			continue;
		else
			config->vhost[config->vhost_len].host_len = strlen( config->vhost[config->vhost_len].host );

		config->vhost_len++;
	}

	if ( tlog.vhost_log_split == TRUE )
	{
		for ( i=config->vhost_len; i--; )
		{
			tmp_logfile = malloc( sizeof(struct logfile_t) * (tlog.logfile_len + 1) );
			if ( tmp_logfile == NULL )
			{
				config_cleanup( config );
				return -1;
			}
			tmp_logfile = memcpy( tmp_logfile, tlog.logfile, sizeof(struct logfile_t) * tlog.logfile_len );
			memset( &(tmp_logfile[tlog.logfile_len]), 0, sizeof(struct logfile_t) );
			free( tlog.logfile );
			tlog.logfile = tmp_logfile;

			tlog.logfile[tlog.logfile_len].vhost = &(config->vhost[i]);
			snprintf( tlog.logfile[tlog.logfile_len].log_filename_access, LOG_FILENAME_LEN, "%s/access_%%d%%d%%d_%%d_(%s:%d).log", tlog.path, config->vhost[i].host, config->vhost[i].port );
			snprintf( tlog.logfile[tlog.logfile_len].log_filename_error, LOG_FILENAME_LEN, "%s/error_%%d%%d%%d_%%d_(%s:%d).log", tlog.path, config->vhost[i].host, config->vhost[i].port );

			tlog.logfile_len++;
		}
	}

	/* virtual host setting end */

	return 0;
}
